# OVM report (jump part)
## Jump problem

One of the problems HAVM has, was the jumps and how they were handled. Two main problems occur on compiled programs.

### Ineffective break

The first problem with jumps in HAVM is with the break in an expression, like this example.

```tiger
while 1 do
    print_int ((break,1))
```

Which give an AST of the tree representation like the following :

```graphviz
graph "Ineffective break"
   {
    fontname="Source Sans Pro"
    node [fontname="consolas"]
    node [fontsize=10, shape=none, height=0.25, margin=0]
    
    label="Ineffective break AST"
    nodesep=0
   
    n002 ;
    n002 [label="seq", width=0.30] ;
    n002 -- n003 ;
    n003 [label="label l1"] ;
    n002 -- n004 ;
    n004 [label="cjump ne"] ;
    n004 -- n005 ;
    n005 [label="const 1"] ;
    n004 -- n006 ;
    n006 [label="const 0"] ;
    n004 -- n007 ;
    n007 [label="name l2"] ;
    n004 -- n008 ;
    n008 [label="name l0"] ;
    n002 -- n009 ;
    n009 [label="label l2"] ;
    n002 -- n010 ;
    n010 [label="sxp"] ;
    n010 -- n011 ;
    n011 [label="call"] ;
    n011 -- n012 ;
    n012 [label="name print_int", width=1] ;
    n011 -- n013 ;
    n013 [label="eseq"] ;
    n013 -- n014 ;
    n014 [label="jump"] ;
    n014 -- n015 ;
    n015 [label="name l0"] ;
    n013 -- n016 ;
    n016 [label="const 1"] ;
    n002 -- n017 ;
    n017 [label="jump"] ;
    n017 -- n018 ;
    n018 [label="name l1"] ;
    n002 -- n019 ;
    n019 [label="label l0"] ;
   }
```

In this example, HAVM evaluate the `eseq` by evaluating the jump and the statement after it, but it will after return to the `eseq` and continuing evaluating it by evaluating the `const 1` and returning 1, doing it indefinitely because of the while.

### Ineffective boolean operator

...

```graphviz
graph "Ineffective break"
   {
    fontname="Source Sans Pro"
    node [fontname="consolas"]
    node [fontsize=10, shape=none, height=0.25, margin=0]
    
    label="Ineffective boolean operator AST"
    nodesep=0
    
    n001 ;
    n001 [label="call"]
    n001 -- n002
    n002 [label="name print_int", width=1]
    n001 -- n003
    n003 [label="eseq"]
    n003 -- n004
    n004 [label="seq", width=0.30]
    n004 -- n005
    n005 [label="seq", width=0.30]
    n005 -- n006
    n006 [label="cjump ne"]
    n006 -- n007
    n007 [label="const 0"]
    n006 -- n008
    n008 [label="const 0"]
    n006 -- n009
    n009 [label="name l3"]
    n006 -- n010
    n010 [label="name l4"]
    n005 -- n011
    n011 [label="cjump ne"]
    n011 -- n012
    n012 [label="const 1"]
    n011 -- n013
    n013 [label="const 0"]
    n011 -- n014
    n014 [label="name l0"]
    n011 -- n015
    n015 [label="name l1"]
    n005 -- n016
    n016 [label="cjump ne"]
    n016 -- n017
    n017 [label="const 0"]
    n016 -- n018
    n018 [label="const 0"]
    n016 -- n019
    n019 [label="name l0"]
    n016 -- n020
    n020 [label="name l1"]
    n004 -- n021
    n021 [label="label l0"]
    n004 -- n022
    n022 [label="move", width=0.45]
    n022 -- n023
    n023 [label="temp t0"]
    n022 -- n024
    n024 [label="const 2"]
    n004 -- n025
    n025 [label="jump", width=0.45]
    n025 -- n026
    n026 [label="name l2"]
    n004 -- n027
    n027 [label="label l1"]
    n004 -- n028
    n028 [label="move", width=0.45]
    n028 -- n029
    n029 [label="temp t0"]
    n028 -- n030
    n030 [label="const 3"]
    n004 -- n031
    n031 [label="label l2"]
    
   }
```

### Proposed solutions

To solve those problems, we choose to use exceptions, something the Haskell programming language doesn't have. The idea is that, when dealing with a jump, the program raise an exception. This exception will be catch somewhere where it is possible to evaluate the code of the destination, but avoid returning to what we were evaluating before.

One problem encountered is getting to the destination quickly. A possible way was scanning all the sequences until finding the destination label. But this method is particulary slow.

#### First approach

For the first approach, we use an map which map to a label a destination.

Unlike HAVM, we don't "plain" the sequences of the program. Instead, we use a stack of sequences. Each cell of the stack contains the end of a sequence, after the destination label or the sub-sequence containing it, which has to be evaluated. The advantage of this is we can restore the sub-sequences without losing informations for the traces.

In this exemple, the stack contain two sequences, in red :
- `[C, (seq [D, E])]`
- `[F, G]`

```graphviz
graph {
    fontname="Source Sans Pro"
    node [fontname="consolas"]
    node [fontsize=10, shape=none, height=0.25, margin=0]
    
    n001 [label="seq", width=0.3] ;
    n001 -- n002 ;
    n002 [label="A", width=0.2]
    n001 -- n003
    n003 [label="seq", width=0.3]
    n003 -- n004
    n004 [label="seq", width=0.3]
    n004 -- n005
    n005 [label="jump", width=0.45]
    n005 -- n006
    n006 [label="name l0"]
    n004 -- n007
    n007 [label="B", width=0.2]
    n003 -- n008
    n008 [label="label l0"]
    
    subgraph {
        cluster=true ;
        shape=box
        node [fontcolor="red2"]
        n003 -- n009
        n009 [label="C", width=0.2]
        n003 -- n010
        n010 [label="seq", width=0.3]
        n010 -- n011
        n011 [label="D", width=0.2]
        n010 -- n012
        n012 [label="E", width=0.2]
        n001 -- n013
        n013 [label="F", width=0.2]
        n001 -- n014
        n014 [label="G", width=0.2]
    }
}
```

Note that `D` and `E` are not in their own cell in the stack, because the sequence they belong is not being evaluated.

The exception is caught by the at the beginning of the program's evaluation, and then the corresponding stack is fetched and then evaluated.

The problem of this approach is the `eseq`. The stack doesn't take account of an `eseq`, and will not restore it nor evaluate it, even if the jump doesn't go out of its statements.

#### Second approach

In the second approach, instead of catching the jump at the beginning of the evaluation, it is also caught by the `eseq`.

To do so, instead of having a unique Map, we have one associated to each `eseq`, with a physical equality. Each Map has the labels of the statements in its statement child, excluding those in child `eseq`'s statements. When a jump is caught, if the destination label is not in the `eseq`'s map, it is propagated. The exception is also caught in the beginning of the program, but not propagated in this case.

# Appendice

```tree
seq
  # ...
  label l1
  cjump ne
    const 1
    const 0
    name l2
    name l0
  label l2
  sxp
    call
      name print_int
      eseq
        jump
          name l0
        const 1
    call end
  jump
    name l1
  label l0
  # ...
seq end
```