open Utils

type t = { fp : int; sp : int; temp : int SMap.t }
(** Type for the temporaries *)

type registry = t
(** Another name *)

(** [initialize ~heapsize ~stacksize ()]
    Initialize the temporaries with their initial values for the specials ones
 *)
let initialize ~(heapsize : int) ~(stacksize : int) () : t =
  {
    fp = heapsize + stacksize - 4;
    sp = heapsize + stacksize - 4;
    temp = SMap.singleton "rv" 0;
  }

(** [rstore registry temp v]
    Store a value [v] in a temporary [temp] in [registry]
    If there is a temporary with that name, change it's value
    If not, create a new one with that value
 *)
let rstore (r : t) (name : string) (x : int) : t =
  match name with
  | "fp" | "$fp" -> { r with fp = x }
  | "sp" | "$sp" -> { r with sp = x }
  | _ ->
      let temp = if name = "$v0" then "rv" else name in
      { r with temp = SMap.update temp (fun _ -> Some x) r.temp }

(** [rfetch registry temp]
    Fetch the content of a temporary [temp] of [registry]
    Raise Not_found exception if there is no temporary with that name
 *)
let rfetch (r : t) (name : string) : int =
  match name with
  | "fp" | "$fp" -> r.fp
  | "sp" | "$sp" -> r.sp
  | _ ->
      let temp = if name = "$v0" then "rv" else name in
      SMap.find temp r.temp

(** [print_register out (temp, x)]
    Print in [out] the name of a register (temporary) [temp] and its content [x]
 *)
let print_register (out : Format.formatter) ((temp, x) : string * int) =
  Format.fprintf out "%s : %d" temp x

(** [print_temp out registry]
    Print all the registers' name and content
 *)
let print_temp (out : Format.formatter) (r : t) =
  Format.fprintf out "Temporaries state :@\nfp : %d@\nsp : %d@\n%a@\n" r.fp r.sp
    (Format.pp_print_seq
       ~pp_sep:(fun fmt () -> Format.fprintf fmt "@\n")
       print_register)
    (SMap.to_seq r.temp)
