open Exceptions
open Result
open Tree

let zeroary f fname floc mem (args : (Tree.expr * Result.t) list) =
  match args with
  | [] -> (
      try f mem
      with RuntimeError (msg, mem) ->
        runtime_error (Format.asprintf "%s" msg) floc mem)
  | _ ->
      runtime_error
        (Format.asprintf "%s : expected 0 argument, given %i" fname
           (List.length args))
        floc mem

let unary f fname floc mem (args : (Tree.expr * Result.t) list) =
  match args with
  | [ (_, Int x) ] -> (
      try f mem x
      with RuntimeError (msg, mem) ->
        runtime_error (Format.asprintf "%s" msg) floc mem)
  | [ (expr, res) ] ->
      runtime_error
        (Format.asprintf "%s : expected integer, given %a" fname Result.print
           res)
        expr.loc mem
  | _ ->
      runtime_error
        (Format.asprintf "%s : expected 1 argument, given %i" fname
           (List.length args))
        floc mem

let binary f fname floc mem (args : (Tree.expr * Result.t) list) =
  match args with
  | [ (_, Int x); (_, Int y) ] -> (
      try f mem x y
      with RuntimeError (msg, mem) ->
        runtime_error (Format.asprintf "%s" msg) floc mem)
  | [ (_, Int _); (expr, res) ] ->
      runtime_error
        (Format.asprintf "%s : second argument expected integer, given %a" fname
           Result.print res)
        expr.loc mem
  | [ (expr, res); (_, Int _) ] ->
      runtime_error
        (Format.asprintf "%s : first argument expected integer, given %a" fname
           Result.print res)
        expr.loc mem
  | _ ->
      runtime_error
        (Format.asprintf "%s : expected 2 argument, given %i" fname
           (List.length args))
        floc mem

let ternary f fname floc mem (args : (Tree.expr * Result.t) list) =
  match args with
  | [ (_, Int x); (_, Int y); (_, Int z) ] -> (
      try f mem x y z
      with RuntimeError (msg, mem) ->
        runtime_error (Format.asprintf "%s" msg) floc mem)
  | [ (_, Int _); (_, Int _); (expr, res) ] ->
      runtime_error
        (Format.asprintf "%s : third argument expected integer, given %a" fname
           Result.print res)
        expr.loc mem
  | [ (_, Int _); (expr, res); (_, _) ] ->
      runtime_error
        (Format.asprintf "%s : second argument expected integer, given %a" fname
           Result.print res)
        expr.loc mem
  | [ (expr, res); (_, _); (_, _) ] ->
      runtime_error
        (Format.asprintf "%s : first argument expected integer, given %a" fname
           Result.print res)
        expr.loc mem
  | _ ->
      runtime_error
        (Format.asprintf "%s : expected 3 argument, given %i" fname
           (List.length args))
        floc mem
