%{

open Tree

%}


/* Keywords */
%token CST
%token NAME
%token TEMP
%token BINOP
%token MEM
%token CALL
%token CALLEND
%token ESEQ
%token MOVE
%token SXP
%token JUMP
%token CJUMP
%token SEQ
%token SEQEND
%token LABEL

/* Operators */
%token ADD
%token SUB
%token MUL
%token DIV
%token MOD

%token ULT
%token UGT
%token ULE
%token UGE
%token LT
%token GT
%token LE
%token GE
%token NEQ
%token EQ

%token <string> TOK_id
%token <string> TOK_const
%token <string> STRING

%token EOF


/* entry points */
%start <stmt list> program

%%

%public %inline loc(X):
x=X { {loc=($startpos,$endpos); payload=x} }

program:
  p=list(stmt) EOF {p}

stmt:
  | s=loc (rawstmt) {s}

rawstmt:
  | LABEL l=TOK_id                   { Label l }
  | LABEL l=TOK_id s=STRING          { Litteral (l,s) }
  | MOVE e1=expr e2=expr             { Move (e1,e2) }
  | SXP e=expr                       { Sxp e }
  | SEQ l=list(stmt) SEQEND          { Seq l }
  | CJUMP rop=relop e1=expr e2=expr NAME l1=TOK_id NAME l2=TOK_id { Cjump (rop,e1,e2,l1,l2)}
  | JUMP e=expr labs=list(TOK_id)    { Jump (e,labs) }

expr:
  | e = loc (rawexpr) {e}

rawexpr:
  | CST c=TOK_const                      { Cst c }
  | TEMP id=TOK_id                       { Temp id }
  | MEM e=expr                           { Mem e }
  | BINOP b=bop e1=expr e2=expr          { Binop(b,e1,e2) }
  | NAME l=TOK_id                        { Name l }
  | CALL f=expr args=list(expr) CALLEND  { Call (f,args) }
  | ESEQ s=stmt e=expr                   { Eseq (s,e) }

bop:
  | ADD {Add}
  | MUL {Mul}
  | SUB {Sub}
  | DIV {Div}
  | MOD {Mod}

relop:
  | EQ { Eq }
  | NEQ { Neq }
  | LT { LT }
  | GT { GT }
  | LE { LE }
  | GE { GE }
  | ULT { ULT }
  | UGT { UGT }
  | ULE { ULE }
  | UGE { UGE }
