open Tree
open Utils

(** stdFun
 *  Predefined functions' map
 *)
let stdFun =
  SMap.of_list
    [
      ("chr", StdChar.chr);
      ("concat", StdString.strconcat);
      ("exit", StdSystem.exit);
      ("flush", StdSystem.flush);
      ("getchar", StdSystem.getchar);
      ("init_array", StdSystem.init_array);
      ("malloc", StdSystem.malloc);
      ("not", StdInt.std_not);
      ("ord", StdChar.ord);
      ("print", StdPrint.print);
      ("print_err", StdPrint.print_err);
      ("print_int", StdPrint.print_int);
      ("size", StdString.size);
      ("strcmp", StdString.strcmp);
      ("streq", StdString.equals);
      ("stringEqual", StdString.equals);
      ("substring", StdString.substring);
    ]

(** of_label
 *  Give the function corresponding to a label
 *)
let of_label (name : label) =
  try SMap.find name stdFun
  with Not_found ->
    failwith
      (Format.sprintf "%s not in standard library or not implemented yet" name)

(** in_stdlib
 *  Check if a label correspond to a function in predefined functions
 *)
let in_stdlib (name : label) : bool = SMap.exists (fun x _ -> x = name) stdFun
