open Exceptions
open Result

(** chr * Predefined function chr * Return the one character long string
    containing the character which code is code * If code does not belong to the
    range [0..255], raise a runtime `error: chr: character out of range` *)
let chr =
  Function.unary
    (fun mem x ->
      if 0 <= x && x < 256 then
        let nmem, addr = Memory.mreserve mem 2 in
        (Memory.mstore_seq nmem addr (List.to_seq [ x; 0 ]), Int addr)
      else
        raise
          (RuntimeError
             (Format.sprintf "chr : character %i out of range [[0,255]]" x, mem)))
    "chr"

(** ord * Predefined function ord * Return the ascii code of the first character
    in string and -1 if the given string is empty *)
let ord =
  Function.unary
    (fun mem addr ->
      try (mem, Int (Memory.mfetch mem addr))
      with Failure msg -> raise (RuntimeError (msg, mem)))
    "ord"
