open Exceptions
open Result

(** exit * Predefined function exit * Exit the program with exit code given *)
let exit = Function.unary (fun mem x -> raise (ExitException (x, mem))) "exit"

(** flush * Predefined function flush * Flush the output buffer *)
let flush =
  Function.zeroary
    (fun mem ->
      Format.fprintf !Vmio.out "%!";
      (mem, Nothing))
    "flush"

(** getchar * Predefined function getchar * Read a character on input. Return an
    empty string on an end of file *)
let getchar =
  Function.zeroary
    (fun mem ->
      match In_channel.input_char !Vmio.input with
      | Some c ->
          let mem', addr = Memory.mreserve mem 8 in
          let mem'' = Char.code c |> Memory.mstore mem' addr in
          (Memory.mstore mem'' (addr + 4) 0, Int addr)
      | None ->
          let mem', addr = Memory.mreserve mem 4 in
          (Memory.mstore mem' addr 0, Int addr))
    "getchar"

(** init_array * Predefined function init_array * Initialize an array of a given
    size with a given value and return its address *)
let init_array =
  Function.binary
    (fun mem size value ->
      let mem', addr = Memory.mreserve mem (4 * size) in
      (Memory.mstore_seq mem' addr (Seq.init size (fun _ -> value)), Int addr))
    "init_array"

(** malloc * Predefined function malloc * Reserve a segment of memory of a given
    size in bytes and return its address *)
let malloc =
  Function.unary
    (fun mem size ->
      let mem', addr = Memory.mreserve mem size in
      (mem', Int addr))
    "malloc"
