open Ovm

let total = ref 0
let pass = ref 0

let test (name : string) (result : bool) =
  incr total;
  Format.printf "Test %s " name;
  if result then (
    incr pass;
    Terminal.ok ())
  else Terminal.ko ()

let test_memory () =
  let mem = ref (Memory.initialize ~heapsize:65536 ~stacksize:16384 []) in
  let tmp, x = Memory.mreserve !mem 5 in
  mem := tmp;
  test "mreserve addr" (x = 0);
  test "hp = 2" (!mem.heap.hp = 2);
  let tmp, res = Memory.add_str !mem "hello" in
  mem := tmp;
  test "(add_str \"hello\") returned address" (res = 2 * Memory.sizeOfInt);
  let tmp, res = Memory.add_str !mem "world !" in
  mem := tmp;
  test "(add_str \"world !\") returned address" (res = 8 * Memory.sizeOfInt);
  test "good hp value" (!mem.heap.hp = 16);
  test "good values stored"
    (List.of_seq (Memory.mfetch_seq !mem (2 * Memory.sizeOfInt) 14)
    = [ 104; 101; 108; 108; 111; 0; 119; 111; 114; 108; 100; 32; 33; 0 ]);
  test "fetch_str_rev on \"hello\""
    (List.of_seq (Memory.fetch_str_rev_seq !mem (2 * Memory.sizeOfInt))
    = List.map Char.chr [ 111; 108; 108; 101; 104 ]);
  test "fetch_str_rev on \"world !\""
    (List.of_seq (Memory.fetch_str_rev_seq !mem (8 * Memory.sizeOfInt))
    = List.map Char.chr [ 33; 32; 100; 108; 114; 111; 119 ]);
  mem :=
    Memory.mstore_seq !mem (8 * Memory.sizeOfInt)
      (List.to_seq [ 104; 101; 108; 108; 111 ]);
  test "mstore_seq give replace \"world !\" by \"hello !\""
    (Memory.fetch_str !mem (8 * Memory.sizeOfInt) = "hello !");
  test "fetching in the last address doesn't raise an error"
    (try
       ignore (Memory.mfetch_seq !mem 81912 2 : int Seq.t);
       true
     with _ -> false);
  test "fetching after the last address raise a Failure error"
    (try
       ignore (Memory.mfetch_seq !mem 81912 3 : int Seq.t);
       false
     with
    | Failure msg -> msg = "Can't fetch values of size 12 at address 81912"
    | _ -> false);
  let loc = (Lexing.dummy_pos, Lexing.dummy_pos) in
  let tmp, r =
    StdSystem.init_array loc !mem
      [
        ({ payload = Cst "5"; loc }, Int 5);
        ({ payload = Cst "42"; loc }, Int 42);
      ]
  in
  mem := tmp;
  test "Test hp after init_array" (!mem.heap.hp = 21);
  test "Test array content"
    (List.of_seq (Memory.mfetch_seq !mem (Result.to_int r) 6)
    = [ 42; 42; 42; 42; 42; 0 ]);
  Format.printf "%i/%i test passed\n" !pass !total
